

#include <SPI.h>
#include <avr/wdt.h>            // library for default watchdog functions
#include <avr/interrupt.h>      // library for interrupts handling
#include <avr/sleep.h>          // library for sleep
#include <avr/power.h>          // library for power control
#include "src/Adafruit_GFX/Adafruit_GFX.h"
#include "src/Adafruit_SSD1306/Adafruit_SSD1306.h"
#include "src/IQS269/IQS269.h"
#include "src/IQS269/IQS269A_init.h"

/*** Defines ***/
//For serial debugging define DEBUG
//#define DEBUG

// AZP985A02 Requirements
#define IQS_RDY 2
#define IQS_ADR 0x44
#define BS0     A0
#define IO_1    A1
#define BS1     A2
#define BS2     A3
#define BTN0    3
#define BTN1    4

// OLED Requirements
#define OLED_GPIO   8
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13

// Coordinates
#define X_MIDDLE  63
#define Y_MIDDLE  31

#define BUTTON  20

/*** Types ***/

typedef union
{
  uint8_t modeByte;
  struct{
    uint8_t dyC:1;
    uint8_t wear:1;
    uint8_t heat:1;
    uint8_t ref:1;
    uint8_t reservedForPrevious:4;  
  }current;
  struct{
    uint8_t reservedForCurrent:4;
    uint8_t dyC:1;
    uint8_t wear:1;
    uint8_t heat:1;
    uint8_t ref:1;
  }previous;
} Mode_t;

typedef union
{
  uint8_t powerByte;
  struct{
    bool np:1;
    bool lp:1;
    bool ulp:1;
    bool reserved;
  }mode;
} Power_t;

typedef union
{
  uint8_t bsByte;
  struct{
    bool _0:1;
    bool _1:1;
    bool _2:1;
    bool prev0:1;
    bool prev1:1;
    bool prev2:1;
    bool _8button:1;
    bool _2button:1;
  }bs;
} BoardSelect_t;
    
/*** Macros ***/
//#define mode (*(Mode_t *) &GPIOR0)

Mode_t mode;
Power_t power;
BoardSelect_t board;

/*** Instances ***/
IQS269 iqs; // Creat an IQS269 instance

Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

/*** Variables ***/
unsigned int firstMODETime = 0;
unsigned int buttonMODETimer = 0;
bool buttonMODEstate = false;

unsigned int firstBTN1Time = 0;
unsigned int buttonBTN1Timer = 0;
bool buttonBTN1state = false;

bool wearState = false;
bool refState = false;
bool heatState = false;

void setup()
{
  // Start Serial communication.
  //#ifdef DEBUG
  //Serial.begin(115200);
  //Serial.println("Serial debugging");
  //#endif
  
  //Enable power to VBAT
  pinMode(OLED_GPIO, OUTPUT);
  digitalWrite(OLED_GPIO, HIGH);
  delay(500);

  // Start OLED.
  display.begin(SSD1306_SWITCHCAPVCC);
  //display.invertDisplay(true);
  display.display();
  delay(3000);
  display.clearDisplay();

  //Initialize BS0 (used for board selection)
  pinMode(BS0, INPUT);
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  //Initialize BS1 (used for board selection)
  pinMode(BS1, INPUT);
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  //Initialize BS2 (used for board selection)
  pinMode(BS2, INPUT);
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011

  //Initialize BTN0 (used for Mode select)
  pinMode(BTN0, INPUT);
  buttonMODEstate = digitalRead(BTN0);
  //Initialize BTN1 (used for Function select)
  pinMode(BTN1, INPUT);
  buttonBTN1state = digitalRead(BTN1);
  
  //Initialize IO_1 (used for HEAT)
  pinMode(IO_1, OUTPUT);
  digitalWrite(IO_1, LOW);   //HEAT => off

  //#ifdef DEBUG
  //Serial.println("IQS269A Setup start");
  //#endif
  
  // Initialize the IQS sensor.
  while(!iqs.begin(IQS_ADR, IQS_RDY));

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif

  if(iqs.checkProductNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("Device is not a IQS269A");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Device connected is    not an IQS269A", 5, 35, 1, WHITE);
    display.display();
    while(1);
  }

  if(iqs.checkSoftwareNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("IQS269A firmware outdated");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Outdated IQS269A FW", 10, 30, 1, WHITE);
    oledDisplayText("IQS269A v1 or later", 10, 45, 1, WHITE);
    display.display();
    while(1);
  }

  if( (board.bs._0) && !(board.bs._1) && (board.bs._2) )
  {
    //continue as normal
    board.bs._2button = true;
  }
  else
  {
    board.bs._2button = false;
    
    display.clearDisplay();
    display.display();
    delay(100);

    oledDisplayText("2 Button    board   required!", 10, 10, 2, WHITE);
    display.display();
    delay(1000);
  }
  
  //Write MM
  iqs.writeMM(RESTART);

  //Enable the partial Hall UI
  iqs.enableHallUI(RESTART);

  // Acknowledge Reset.
  iqs.acknowledgeReset(RESTART);
  
  // Autone after making settings.
  iqs.autoTune(STOP);
  delay(500);
  
  // Do a startup routine

  oledStartupRoutine_2();

  // Display that initialization is complete and sensor is ready.
  while(!iqs.waitForReady());
  // Acknowledge Reset.
  iqs.acknowledgeReset(STOP);

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif
  
  display.clearDisplay();

  //Initialize zero states
    mode.current.dyC = false;
    mode.current.wear = false;
    mode.current.heat = false;
    mode.current.ref = false;

    mode.previous.dyC = false;
    mode.previous.wear = false;
    mode.previous.heat = false;
    mode.previous.ref = false;

    wearState = false;
    refState = false;
    
  //Always start in NP
    power.mode.np = true;
    power.mode.lp = false;
    power.mode.ulp = false;
    
    //#ifdef DEBUG
    //Serial.println("Normal Power mode");
    //#endif

    // Update OLED.
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE);

    oledDisplayText("Mode: Default ", 25, 54, 1, WHITE);
    
} //setup   

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void loop()
{
  
  signed int ch1CS = 0;
  signed int ch1LTA = 0;
  signed int ch1Delta = 0;
  signed int ch2CS = 0;
  signed int ch2LTA = 0;
  signed int ch2Delta = 0;
  signed int ch4CS = 0;
  signed int ch4LTA = 0;
  signed int ch4Delta = 0;
  signed int ch5Delta = 0;
  signed int ch7Delta = 0;
  
  //Buffer last board select
  board.bs.prev0 = board.bs._0;
  board.bs.prev1 = board.bs._1;
  board.bs.prev2 = board.bs._2;
  
  //Read BS0,BS1,BS2 (used for board selection)
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011

//    if(iqs.checkProductNum(RESTART) == true)
//    {
//    wdt_reset();
//    wdt_enable(WDTO_8S);
//    }
//    else
//    {
//   //#ifdef DEBUG
//    //Serial.println("Device is not a IQS269A");
//    //#endif
//    display.clearDisplay();
//    display.display();
//    delay(100);
//    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
//    oledDisplayText("No IQS269A connected", 5, 35, 1, WHITE);
//    display.display();
//    if(iqs.checkProductNum(RESTART) == true)
//    {
//    wdt_reset();
//    setup();
//    }
//    wdt_enable(WDTO_15MS);
//    }
  
  if( (board.bs._0) && (!board.bs._1) && (board.bs._2) )
  {
    //continue as normal
    board.bs._2button = true;
  }
  else
  {
    board.bs._2button = false;

    do
    {
      display.clearDisplay();
      display.display();
      delay(100);
  
      oledDisplayText("2 Button    board   required!", 10, 10, 2, WHITE);
      display.display();
      delay(1000);
      
      //Buffer last board select
      board.bs.prev0 = board.bs._0;
      board.bs.prev1 = board.bs._1;
      board.bs.prev2 = board.bs._2;
      
      //Read BS0,BS1,BS2 (used for board selection)
      board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
      board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
      board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011
    }while(board.bs._1);
        
    setup();
  }
  
  //Read BTN0 (used for Mode select)
  buttonMODEstate = digitalRead(BTN0);
  //Read BTN1 (used for Function select)
  buttonBTN1state = digitalRead(BTN1);
  
  // Read sensor information.
  while(!iqs.waitForReady());
  iqs.updateSystemFlags(RESTART);
  iqs.updateEvents(RESTART);
  iqs.updateProxByte(RESTART);
  //iqs.updateTouchByte(RESTART);
  iqs.updateProxdirectionByte(RESTART);
  
  iqs.updateTouchByte(RESTART);
  iqs.updateDeepByte(RESTART);
  
  //iqs.counts.ch4 = iqs.readChannelCounts(CH_4, RESTART);
  ch4CS = iqs.readChannelCounts(CH_4, RESTART);
  //iqs.lta.ch4 = iqs.readChannelLta(CH_4, RESTART);
  ch4LTA = iqs.readChannelLta(CH_4, RESTART);
  //Below delta is not reliable: LTA tracking adjustment occurs only after this Delta calculation on-chip
  //ch4Delta = iqs.readChannelDelta(CH_4, RESTART);
  //ch4Delta = (signed)(iqs.lta.ch4 - iqs.counts.ch4);
  ch2CS = iqs.readChannelCounts(CH_2, RESTART);
  ch2LTA = iqs.readChannelLta(CH_2, RESTART);
  ch1CS = iqs.readChannelCounts(CH_2, RESTART);
  ch1LTA = iqs.readChannelLta(CH_2, RESTART);
  ch4Delta = (signed)(ch1LTA - ch1CS);
  ch1Delta = (signed)(ch4LTA - ch4CS);
  ch2Delta = (signed)(ch2LTA - ch2CS);
  ch5Delta = (signed)(iqs.readRefChannelDelta(CH_4, RESTART));
  ch7Delta = (signed)(iqs.readChannelDelta(CH_7, STOP));
  oledBar(0, ch7Delta);
  

//______________________________________start 2button demo
    //ULP CH0 + CH4 wear + CH5 ref
    //CH0
    if(!mode.current.dyC)
    {
      /*----------------------------------------Evaluate deep touch, touch or prox------------------------------------*/
      //evaluate deep touch draw 2 rectangles
      if(iqs.deep.ch4)
      {
      oledDisplayText("1", 33, 25, 2, WHITE);
      display.drawRoundRect(20, 14, BUTTON+15, BUTTON+15, 5, WHITE);
      display.drawRoundRect(22, 16, BUTTON+11, BUTTON+11, 5, WHITE);
      oledDisplayText("2", 73, 25, 2, WHITE);
      display.drawRoundRect(60, 14, BUTTON+15, BUTTON+15, 5, WHITE);
      display.drawRoundRect(62, 16, BUTTON+11, BUTTON+11, 5, WHITE);
      }
      
      //evaluate touch draw 1 rectangle
      else if(iqs.touch.ch4)
      {
      oledDisplayText("1", 33, 25, 2, WHITE);
      display.drawRoundRect(20, 14, BUTTON+15, BUTTON+15, 5, WHITE);
      oledDisplayText("2", 73, 25, 2, WHITE);
      display.drawRoundRect(60, 14, BUTTON+15, BUTTON+15, 5, WHITE);
      }

      //evaluate proximity display number of channel
      else if(iqs.prox.ch4)
      {
      // Update OLED.
      oledClearDeep();
      oledDisplayText("1", 33, 25, 2, WHITE);
      oledDisplayText("2", 73, 25, 2, WHITE);
      }

      //if no prox, touch or deep touch, clear display
      else
      {
      display.fillRect(20, 14, BUTTON+15, BUTTON+15, BLACK);
      display.fillRect(60, 14, BUTTON+15, BUTTON+15, BLACK);
      }
      /*----------------------------------End prox/deep touch/touch evaluation------------------------------------*/

      // Update mode flags.
      mode.current.wear = true;
      if(!wearState)
      {
        wearState = true;
  
        //#ifdef DEBUG
        //Serial.println("Wear CH4");
        //#endif
        
        // Update OLED.
        oledDisplayText("Wear", 101, 24, 1, WHITE);
      }
  
      oledClearWearDelta();
      oledDisplayText(String(ch4Delta), 101, 34, 1, WHITE);
      if(refState)
      {
        oledClearRefDelta();
        oledDisplayText(String(ch5Delta), 25, 2, 1, WHITE);
      }
    }
    else if(!mode.current.dyC && !iqs.prox.ch4)
    {
      oledClearAllButtons();
      oledClearWear();
      oledClearRefDelta();
      oledClearWearDelta();
      // Update mode flags.
      mode.current.wear = false;
      wearState = false;
    }

    //Dynamic_C
    else if(mode.current.dyC && iqs.prox.ch0)
    {
      if(iqs.prox.ch1)
      {
        oledBarT(25, ch1Delta);
        oledDisplayText("MC", 30, 35, 1, WHITE);
      }
      if(iqs.prox.ch2)
      {
        oledBarT(70, ch2Delta);
        oledDisplayText("SC", 75, 35, 1, WHITE);
      }
             
      // Update mode flags.
      mode.current.wear = true;
      if(!wearState)
      {
        wearState = true;
  
        //#ifdef DEBUG
        //Serial.println("Wear CH4");
        //#endif
        
        // Update OLED.
        oledDisplayText("Wear", 101, 24, 1, WHITE);
      }
      oledClearWearDelta();
      oledDisplayText(String(ch4Delta), 101, 34, 1, WHITE);
      if(refState)
      {
        oledClearRefDelta();
        oledDisplayText(String(ch5Delta), 25, 2, 1, WHITE);
      }
    } 
    else if(mode.current.dyC && !iqs.prox.ch0)
    {
      if(iqs.dir.ch1 && wearState)
      {
        wearState = false;
          
        // Update mode flags.
        mode.current.wear = false;
        oledClearAllButtons();
        oledClearWear();
        oledClearWearDelta();
        oledClearRefDelta();
      }
    }
  
    //BTN1
    if(!buttonBTN1state) //BTN1 => REF -> REF+HEAT -> REF -> OFF
    {          
      if(!buttonBTN1state && buttonBTN1Timer == 0)
      { 
        // Reset BTN1 timer.
        firstBTN1Time = millis();
        delay(1);
  
        //#ifdef DEBUG
        //Serial.println("BTN1 => REF");
        //#endif
      }
  
      buttonBTN1Timer = millis() - firstBTN1Time;
      
      if (buttonBTN1Timer > 1000 && !buttonBTN1state) //longer than 1sec hold
      {     
        if(!refState && !heatState)
        {
          //Activate the Ref_ch
          while(!iqs.waitForReady());
          iqs.setRefChannel(CH_5, CH_4, STOP);
          refState = true;
  
          //#ifdef DEBUG
          //Serial.println("REF activated");
          //#endif
          
          // Display REF on OLED.
          oledDisplayText("REF", 2, 2, 1, WHITE);

          buttonBTN1Timer = 0;
          
          // Update mode flags.
          mode.current.ref = true;
        }
        else if(refState && !heatState)
        {
          digitalWrite(IO_1, HIGH);   //HEAT => on 
          heatState = true;
  
          //#ifdef DEBUG
          //Serial.println("HEAT activated");
          //#endif
  
          // Display HEAT on OLED.
          oledDisplayText("HEAT", 101, 2, 1, WHITE);
  
          buttonBTN1Timer = 0;

          // Update mode flags.
          mode.current.heat = true;
        }
        else if(refState && heatState)
        {
          digitalWrite(IO_1, LOW);   //HEAT => off
          heatState = false;
  
          //#ifdef DEBUG
          //Serial.println("HEAT de-activated");
          //#endif
  
          // Clear HEAT on OLED.
          oledClearFunc();
  
          buttonBTN1Timer = 0;

          // Update mode flags.
          mode.current.heat = false;
        }
        else
        {
          //De-activate the Ref_ch
          while(!iqs.waitForReady());
          iqs.clearRefChannel(CH_5, CH_4, STOP);
          refState = false;
  
          //#ifdef DEBUG
          //Serial.println("REF de-activated");
          //#endif
          
          // Clear REF on OLED.
          oledClearRef();
  
          buttonMODETimer = 0;
        }
      }
    }
    else if(buttonBTN1state)
    {
      buttonBTN1Timer = 0;
    }
  
    //MODE
    if(!buttonMODEstate) //MODE => DYNAMIC_C
    {     
      if(!buttonMODEstate && buttonMODETimer == 0)
      { 
        // Reset BTN0 timer.
        firstMODETime = millis();
        delay(1);
  
        //#ifdef DEBUG
        //Serial.println("MODE => DYNAMIC_C");
        //#endif
      }
  
      buttonMODETimer = millis() - firstMODETime;
      
      if (buttonMODETimer > 1000 && !buttonMODEstate)  //longer than 1sec hold
      {
          //reset reference state when mode is switched
          while(!iqs.waitForReady());
          iqs.clearRefChannel(CH_5, CH_4, STOP);
          refState = false;
 
          // reset heat state when mode is switched 
          oledClearRef();
          digitalWrite(IO_1, LOW);   //HEAT => off
          heatState = false;  
          // Clear HEAT on OLED.
          oledClearFunc();
  
          buttonBTN1Timer = 0;

          // Update mode flags.
          mode.current.heat = false;
          
        if(!mode.current.dyC)
        {
          //Activate the Dynamic_C
          mode.current.dyC = true;
  
          //#ifdef DEBUG
          //Serial.println("DYNAMIC_C activated");
          //#endif
          
          // Display ModeName on OLED.
          oledClearModeName();
          oledDisplayText("Mode: Dynamic C", 25, 54, 1, WHITE);

          oledClearAllButtons();
          
          buttonMODETimer = 0;
        }
  
        else if(mode.current.dyC)
        {
          //De-activate the Dynamic_C
          mode.current.dyC = false;
  
          //#ifdef DEBUG
          //Serial.println("Dynamic C de-activated");
          //#endif
          
          // Display ModeName on OLED.
          oledClearModeName();
          oledDisplayText("Mode: Default ", 25, 54, 1, WHITE);

          oledClearAllButtons();
  
          buttonMODETimer = 0;
        }
      }
    }
    else if(buttonMODEstate)
    {
      buttonMODETimer = 0;
    }
//______________________________________end 2button

//______________________________________start 8button demo
  if(!board.bs._2button)
  {
    do
    {
      display.clearDisplay();
      display.display();
      delay(100);
  
      oledDisplayText("2 Button    board   required!", 10, 10, 2, WHITE);
      display.display();
      delay(1000);
      
      //Buffer last board select
      board.bs.prev0 = board.bs._0;
      board.bs.prev1 = board.bs._1;
      board.bs.prev2 = board.bs._2;
      
      //Read BS0,BS1,BS2 (used for board selection)
      board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
      board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
      board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011
    }while(board.bs._1);
        
    setup();
  }
//______________________________________end 8button demo

  //LP
  if (iqs.eventFlags.power && iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = true;
    power.mode.ulp = false;
    
    //#ifdef DEBUG
    //Serial.println("LP Mode");
    //#endif
    
    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("LP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE);
  }

  //ULP
  else if (iqs.eventFlags.power && iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = false;
    power.mode.ulp = true;
    
    //#ifdef DEBUG
    //Serial.println("ULP Mode");
    //#endif
    
    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 20, 10, 2, WHITE);
    oledDisplayText("ULP", X_MIDDLE-13, 2, 1, BLACK);
    oledDisplayText(" xuA", X_MIDDLE+2, 2, 1, WHITE);
  }

  //NP
  if (!iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = true;
    power.mode.lp = false;
    power.mode.ulp = false;
    
    //#ifdef DEBUG
    //Serial.println("NP mode");
    //#endif
    
    // Update OLED.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE); 
  }

  //Single display update at the end
  display.display();
}//main loop

/* @brief Displays the Delta parameter on the OLED display at x position. */
void oledBarT(uint8_t x, int16_t delta)
{
  if(mode.current.dyC)
       display.fillRect(x, 10, 19, 45, BLACK);
//    //center line
      display.drawLine(x, Y_MIDDLE, x+19, Y_MIDDLE, WHITE);
//  
//    //clear delta
      display.fillRect(x, 40, 23, 15, BLACK);
      oledDisplayText(String(abs(delta)), x, 45, 1, WHITE);
    
  if(delta/3 > 21) //Just to control the max
  {
     delta = 21*3;
  }
  else if(delta/3 < -21)
  {
     delta = -21*3;
  }
  
  if(delta > 0)
  {
    display.fillRect(x+6, Y_MIDDLE-(delta/3), 8, (delta/3), WHITE);
    if(delta > 10)
    oledDisplayText("+", x, Y_MIDDLE-10, 1, WHITE);
  }
  else
  {
    display.fillRect(x+6, Y_MIDDLE, 8, (abs(delta)/3), WHITE);
    if(delta < -10)
    oledDisplayText("-", x, Y_MIDDLE+5, 1, WHITE);
  }
}

void oledBar(uint8_t x, int16_t delta)
{
    //clear bar area
    display.fillRect(x, 10, 19, 54, BLACK);
    //center line
    display.drawLine(x, Y_MIDDLE, x+19, Y_MIDDLE, WHITE);
  
    //clear delta
    display.fillRect(x, 54, 23, 64, BLACK);
    oledDisplayText(String(abs(delta)), x, 54, 1, WHITE);
    
  if(delta/3 > 21) //Just to control the max
  {
     delta = 21*3;
  }
  else if(delta/3 < -21)
  {
     delta = -21*3;
  }
  
  if(delta > 0)
  {
    display.fillRect(x+6, Y_MIDDLE-(delta/3), 8, (delta/3), WHITE);
    if(delta > 10)
    oledDisplayText("+", x, Y_MIDDLE-10, 1, WHITE);
  }
  else
  {
    display.fillRect(x+6, Y_MIDDLE, 8, (abs(delta)/3), WHITE);
    if(delta < -10)
    oledDisplayText("-", x, Y_MIDDLE+5, 1, WHITE);
  }
}

/*@brief Clears Deep touch display*/

void oledClearDeep()
{
  display.fillRect(20, 14, BUTTON+15, BUTTON+15, BLACK);
  display.fillRect(60, 14, BUTTON+15, BUTTON+15, BLACK);
}

/* @brief Clears the OLED display. */
void oledClear()
{
  display.clearDisplay();
}

/* @brief Clears the OLED CH1 display. */
void oledClearCH1()
{
  display.fillRect(20, 14, BUTTON, BUTTON, BLACK);
}

/* @brief Clears the OLED CH2 display. */
void oledClearCH2()
{
  display.fillRect(40, 14, BUTTON, BUTTON, BLACK);
}

/* @brief Clears the OLED Button area display. */
void oledClearAllButtons()
{
  display.fillRect(20, 14, 4*BUTTON, 2*BUTTON, BLACK);
}

/* @brief Clears the OLED Wear display. */
void oledClearWear()
{
  display.fillRect(101, 24, 27, 10, BLACK);
}

/* @brief Clears the OLED Mode name display. */
void oledClearModeName()
{
  display.fillRect(25, 54, 103, 10, BLACK);
}

/* @brief Clears the OLED WearDelta display. */
void oledClearWearDelta()
{
  display.fillRect(101, 34, 27, 10, BLACK);
}

/* @brief Clears the OLED RefDelta display. */
void oledClearRefDelta()
{
  display.fillRect(25, 0, 20, 10, BLACK);
}

/* @brief Clears the OLED Func display. */
void oledClearFunc()
{
  display.fillRect(100, 2, 28, 20, BLACK);
}

/* @brief Clears the OLED REF display. */
void oledClearRef()
{
  display.fillRect(2, 2, 25, 10, BLACK);
}

/* @brief Clears the OLED PM display. */
void oledClearPM()
{
  display.fillRect(X_MIDDLE-15, 0, 40, 12, BLACK);
}

/* @brief Clears the OLED Gesture display. */
void oledClearGesture()
{
  display.fillRect(101, 30, 27, 20, BLACK);
}

void oledStartupRoutine_2(void)
{

  display.clearDisplay();
  display.display();
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  //PM demonstrations
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("LP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 20, 10, 2, WHITE);
    oledDisplayText("ULP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // MODE demonstration.
    oledDisplayText("Mode: Default ", 25, 54, 1, WHITE);
    display.display();
    delay(500);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // Display REF on OLED.
    oledDisplayText("REF x", 2, 2, 1, WHITE);
    display.display();
    delay(500);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // Display FUNC on OLED.
    oledDisplayText("FUNC", 100, 2, 1, WHITE);  
    display.display();
    delay(500);
    oledClearFunc();
    oledDisplayText("HEAT", 100, 2, 1, WHITE);  
    display.display();
    delay(500);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // CH Demonstration.
    oledDisplayText("1", 33, 25, 2, WHITE);
    display.drawRoundRect(20, 14, BUTTON+15, BUTTON+15, 5, WHITE);
    display.display();
    delay(100);
    oledDisplayText("2", 73, 25, 2, WHITE);
    display.drawRoundRect(60, 14, BUTTON+15, BUTTON+15, 5, WHITE);
    display.display();
    delay(100);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // WEAR demonstration.
    oledDisplayText("Wear", 101, 24, 1, WHITE);
    display.display();
    delay(500);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // MODE demonstration.
    oledClearModeName();
    oledClearAllButtons();
    oledDisplayText("Mode: Dynamic C", 25, 54, 1, WHITE);
    oledDisplayText("1", 33, 20, 1, WHITE);
    oledDisplayText("Rx", 30, 35, 1, WHITE);
    display.drawRoundRect(20, 14, BUTTON+15, BUTTON+15, 5, WHITE);
    oledDisplayText("2", 73, 20, 1, WHITE);
    oledDisplayText("Tx", 70, 35, 1, WHITE);
    display.drawRoundRect(60, 14, BUTTON+15, BUTTON+15, 5, WHITE);
    display.display();
    delay(1000);
    oledClearModeName();
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  // Hall demonstartion.
  oledDisplayText("Hall", 0, 54, 1, WHITE);
  oledDisplayText("+", 0, Y_MIDDLE-10, 1, WHITE);
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
   
  //center line
  display.drawLine(0, Y_MIDDLE, 19, Y_MIDDLE, WHITE);
  display.display();
  delay(100);
  for(int count = 0; count < 20; count++)
  {
    display.fillRect(0, 10, 19, 20, BLACK);
    display.fillRect(6, Y_MIDDLE-(count), 8, (count), WHITE);
    display.display();
    delay(10);
  }
  
    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    }
  
  //clear bar area
  display.fillRect(0, 10, 19, 21, BLACK);
  oledDisplayText("-", 0, Y_MIDDLE+5, 1, WHITE);
  display.display();
  delay(100);

    //exit on button press
    //if button pressed return to main loop
    if(digitalRead(BTN1) == 0)
    {
    //exit startup, return to main loop
    return;
    } 
  
  for(int count = 0; count > -20; count--)
  {
    display.fillRect(6, Y_MIDDLE, 8, abs(count), WHITE);
    display.display();
    delay(10);
  }
  delay(500);
  display.clearDisplay();
  display.display();
  delay(100);
}

void oledDisplayText(String text, unsigned char xCoord, unsigned char yCoord, unsigned char textSize, unsigned char color)
{
  display.setTextSize(textSize);
  display.setTextColor(color);
  display.setCursor(xCoord, yCoord);
  display.println(text);
}
